-- All code written and maintained by Yewchi 
-- zyewchi@gmail.com

-- Rushing to get this into a working Alpha. Most code is functional and 
--    basically extendable, so turning full modular will be easy later on, 
--    after some abstraction.

DEBUG = false
VERBOSE = DEBUG and true

--- Initialization --
--
------- Filenames --
--
AUDIOQS_ROOT = 		"Interface/AddOns/AudioQs/"
SOUNDS_ROOT = 		AUDIOQS_ROOT.."Sounds/"
--
------ /Filenames --

------- Flags --
--
SPELL_TYPE_ABILITY = 								0xFFF0 -- Type of a spell which is to be checked
SPELL_TYPE_AURA = 									0xFFF1

SPEC_NOT_IMPLEMENTED =								0xFFFF -- Nothing exists in AudioQs.lua WTF file

ANY_SPEC_ALLOWED =									0xFFF0 -- For loading Extensions
ONE_SPEC_ALLOWED = 									0xFFF1
--
------ /Flags --

------- Table key references --
--
SPELL_SPELL_NAME = 1
SPELL_CHARGES = 2
SPELL_DURATION = 3
SPELL_EXPIRATION = 4
SPELL_UNIT_ID = 5
SPELL_SPELL_TYPE = 6
SPELL_PROMPT_INDEX = 7 -- TODO Defunct

UNIT_AURA_NAME = 1
UNIT_AURA_COUNT = 3
UNIT_AURA_DURATION = 5
UNIT_AURA_EXPIRATION = 6
UNIT_AURA_UNIT_ID = 7
UNIT_AURA_SPELL_ID = 10
UNIT_AURA_TIME_MOD = 15 -- always = 1. For prelim confirmation of a valid aura

SPEC_INFO_NAME = 2
UNIT_CLASS_NUM = 3
--
------ /Table key references --

------- Static vals --
--
local MAX_ERRORS_PER_SESSION = 3
local DUPLICATE_ERROR_GARBAGE_TIMER = 3.0

AURA_ALWAYS_IS_VALUE = 1 -- using value #15: timeMod

PLAYER_GUID = UnitGUID("player")
--
------- /Static vals --

------- Return values --
--
SPELLS_SPELL_NOT_LISTED = 							0xFFF0
--
------ /Return values --

------- Error Codes --
--
ERR_UNKNOWN =										0xEFF0
ERR_INVALID_ARGS = 									0xEFF1
ERR_UNKNOWN_SPELL_AS_ARGUMENT = 					0xEFF2
ERR_UNKNOWN_EVENT_AS_ARGUMENT =						0xEFF3
ERR_INVALID_AURA_DATA =								0xEFF4
ERR_INVALID_SOUND_DATA = 							0xEFF5
ERR_INVALID_CONDITIONAL_RESULT =					0xEFF6
--
------ /Error Codes --

------- Default Msgs --
--
audioQsSpecifier = "|cff50C0F0[AudioQs]".."|r "
infoSpecifier = "|cff50A0FF<INFO>".."|r: "
debugSpecifier = "|cffF080F0<DEBUG>".."|r: "
errSpecifier = "|cffF08050<ERR>".."|r: "
ERR_MSGS = { 
	[ERR_UNKNOWN] =
		audioQsSpecifier..
		errSpecifier..
		"#"..ERR_UNKNOWN.." "..
		"Unknown error.",
	[ERR_INVALID_ARGS] =
		audioQsSpecifier..
		errSpecifier..
		"#"..ERR_INVALID_ARGS.." "..
		"Invalid args.",
	[ERR_UNKNOWN_SPELL_AS_ARGUMENT] = 
		audioQsSpecifier..
		errSpecifier..
		"#"..ERR_UNKNOWN_SPELL_AS_ARGUMENT.." "..
		" An unlisted spellId was passed to a function.",
	[ERR_UNKNOWN_EVENT_AS_ARGUMENT] = 
		audioQsSpecifier..
		errSpecifier..
		"#"..ERR_UNKNOWN_SPELL_AS_ARGUMENT.." "..
		" An unlisted event was passed to a function.",
	[ERR_INVALID_AURA_DATA] = 
		audioQsSpecifier..
		errSpecifier..
		"#"..ERR_INVALID_AURA_DATA.." "..
		" Data passed to a function did not contain a valid aura table.",
	[ERR_INVALID_SOUND_DATA] = 
		audioQsSpecifier..
		errSpecifier..
		"#"..ERR_INVALID_AURA_DATA.." "..
		" Sound data retreived from a segment was neither a number nor a filepath.",
	[ERR_INVALID_CONDITIONAL_RESULT] =
		audioQsSpecifier..
		errSpecifier..
		"#"..ERR_INVALID_CONDITIONAL_RESULT.." "..
		" An invalid result was derived from a segment conditional"
}
extensionColour = "|cffFFA020"
STOP_ERROR_MAX_REPORTS = audioQsSpecifier.." Max errors exceeded. Stopping error reports."
--
------ /Default msgs --

------- AddOn variables --
--
local errorsThisSession = 0
local mostRecentErrorString = nil
local mostRecentErrorTimestamp = 0

local customEventFrameTimestamp = 0
local customEventsRegisteredThisFrame = {}

specId = nil
hushMode = false
--
------ /AddOn variables --
--
-- /Initialization --

--- Funcs --
--
local UpdateSpecializationInfo

-- TODO SLOW
-------------- FreshEventForFrame()
local function FreshEventForFrame(event)
	local currTime = GetTime()
	local frameSeen = customEventFrameTimestamp == currTime
	if not frameSeen then
		customEventFrameTimestamp = currTime
		for k,_ in pairs(customEventsRegisteredThisFrame) do
			customEventsRegisteredThisFrame[k] = false
		end
	end
	if customEventsRegisteredThisFrame[event] ~= true then
		customEventsRegisteredThisFrame[event] = true
		return true
	end
	return false
end

-- Needs automatic arguments string concats, errors need args={val1,val2}
-------------- HandleError()
function HandleError(err, parent, child)
	if not DEBUG and errorsThisSession >= MAX_ERRORS_PER_SESSION then 
		return
	end
	
	if type(err) == "table" then 
		if err.code == nil then err.code = ERR_UNKNOWN end
	elseif err == nil then 
		err = {code=ERR_UNKNOWN}
	end
	
	-- Formulate full string --
	local msgGiven = (type(err) == "string" and err or "")
	local errorString = ""
	local funcString = (err.func~=nil and " func:"..err.func or (parent~=nil and parent or "???").."->"..(child~=nil and child or "???"))
	
	if type(err.code) == "number" then 
		local errMsg = ERR_MSGS[err.code]
		errorString = (errMsg~=nil and errMsg or "[InvalidErrCode]")..funcString
	else
		errorString = ERR_MSGS[ERR_UNKNOWN].." "..msgGiven.." "..funcString
	end
	
	-- Check error frequency of duplicates is not too high, and print. --
	local currTime = GetTime()
	if not (errorString == mostRecentErrorString and
			currTime < mostRecentErrorTimestamp + DUPLICATE_ERROR_GARBAGE_TIMER) then
		mostRecentErrorString = errorString
		mostRecentErrorTimestamp = currTime
		
		print(errorString)
		
		errorsThisSession = errorsThisSession + 1
		if not DEBUG and errorsThisSession >= MAX_ERRORS_PER_SESSION then
			print(STOP_ERROR_MAX_REPORTS)
		end
	end
end

function ReregisterEvents()
	UpdateSpecializationInfo(nil, nil, "reregister")
end
--
-- /Funcs --

--- Event Funcs --
--
local Frame_SpellUpdateCooldown = CreateFrame("Frame", "Spell Update Charges")
local Frame_CombatLog = CreateFrame("Frame", "Combat Log")
local Frame_UnitAura = CreateFrame("Frame", "Unit Aura")
local Frame_CustomEvents = CreateFrame("Frame", "Custom Events")
local Frame_LoadOrSpecChange = CreateFrame("Frame", "Load or Spec Change")

local function AnyEventsRegistered()
	return Frame_SpellUpdateCooldown:IsEventRegistered("SPELL_UPDATE_COOLDOWN") or Frame_CombatLog:IsEventRegistered("COMBAT_LOG_EVENT_UNFILTERED") or Frame_UnitAura:IsEventRegistered("UNIT_AURA")
end

-------------- RegisterEvents()
local function RegisterEvents()
	Frame_SpellUpdateCooldown:RegisterEvent("SPELL_UPDATE_COOLDOWN")
	Frame_SpellUpdateCooldown:RegisterEvent("SPELL_UPDATE_CHARGES")
	Frame_CombatLog:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	Frame_UnitAura:RegisterEvent("UNIT_AURA")
	GSI_RegisterCustomEvents(Frame_CustomEvents)
if DEBUG then print(audioQsSpecifier..debugSpecifier.."Prompt tracking enabled.") end
end

-------------- UnregisterEvents()
local function UnregisterEvents()
	Frame_CombatLog:UnregisterAllEvents()
	Frame_SpellUpdateCooldown:UnregisterAllEvents()
	Frame_UnitAura:UnregisterAllEvents()
	GSI_UnregisterCustomEvents(Frame_CustomEvents)
if DEBUG then print(audioQsSpecifier..debugSpecifier.."Prompt tracking disabled.") end
end

-------------- LoadSpec() -- self, event, directive
UpdateSpecializationInfo = function(self, event, ...)
	local args = {...}
	if args then
		if args[1] == "reregister" then
			UnregisterEvents() -- Will force the reload of custom events
			RegisterEvents()
		end
	end
	
	newSpecId = GetSpec()
	local eventsWereRegistered = AnyEventsRegistered()
	
	if event == "PLAYER_SPECIALIZATION_CHANGED" and specId == newSpecId then -- PLAYER_SPECIALIZATION_CHANGED can be thrown on other players joining the raid
		return
	end
	
	specId = newSpecId
	local specHasPrompts = SpecHasPrompts(newSpecId)
if DEBUG then print(audioQsSpecifier..debugSpecifier.."Spec #"..newSpecId.." has "..(specHasPrompts == true and "" or "no ").."available prompts.") end
	
	local success, err = pcall(LoadSpecialization)
	if not success then HandleError(err, "UpdateSpecializationInfo()", "LoadSpecialization()") end -- More robust coverage
	
	if specHasPrompts then
if DEBUG then print(audioQsSpecifier..debugSpecifier.."Loaded spec "..specId) end
		if not eventsWereRegistered then -- TODO As event prompts are added, requires an autmatically generated list of frames/events to call for the rules. Event registration needs a more robust process
			RegisterEvents()
		end
	elseif eventsWereRegistered then
if DEBUG then print(audioQsSpecifier..debugSpecifier.."Turning off AudioQs") end
		UnregisterEvents()
	end
end	
Frame_LoadOrSpecChange:RegisterEvent("PLAYER_SPECIALIZATION_CHANGED")
Frame_LoadOrSpecChange:RegisterEvent("PLAYER_LOGIN")
Frame_LoadOrSpecChange:SetScript("OnEvent", function(self, event, ...) UpdateSpecializationInfo(self, event, ...) end)

Frame_CombatLog:SetScript("OnEvent",
	function(self, event, ...)
		local success, err = pcall(ProcessCombatLogForPrompts)
		if not success then HandleError(err, "CombatLogOnEvent", "ProcessCombatLogForPrompts()") end
	end
)

Frame_SpellUpdateCooldown:SetScript("OnEvent",
	function(self, event, ...)
		local success, err = pcall(ProcessSpellCooldownsForPrompts)
		if not success then HandleError(err, "SpellUpdateCooldownOnEvent()", "ProcessSpellCooldownsForPrompts()") end
	end
)

-- Optimize for already completed checks, no further spell prompts possible, etc.
Frame_UnitAura:SetScript("OnEvent",
	function(self, event, ...)
		local unitId = ...
		
		if not UnitIsIncluded(unitId) then
			return
		end
		
		if specId ~= nil then
		-- Abstract into class file funcs
			for n = 1, 40, 1 do
				local aura = {UnitAura(unitId, n, "HELPFUL")}
				if aura[UNIT_AURA_SPELL_ID] ~= nil then 
					local success, err = pcall(FindPromptsFromUnitAura, aura)
					if not success then HandleError(err, "UnitAuraOnEvent()", "FindPromptsFromUnitAura()") break end
				else
					break 
				end
			end
		end
	end
)

Frame_CustomEvents:SetScript("OnEvent",
	function(self, event, ...)
		if not FreshEventForFrame(event) then return end

		--print("Event starting ProcessCustomEventForPrompts() is "..event)
		
		local success, err = pcall(ProcessCustomEventForPrompts, event, ...)
		if not success then HandleError(err, "CustomEventsOnEvent()", "ProcessCustomEventForPrompts()") end
	end
)
--
-- /Event Funcs --