-- All code written and maintained by Yewchi 
-- zyewchi@gmail.com

--~ This serves as both a regular Beast Mastery extension, as well as the main example of an implemented extension.
--~ This file is intended for WoW programmers, and would be overwelming for most others. Please email me if you need specific prompts made, or help writing segment data: zyewchi@gmail.com 

local extName = "BeastMastery"
local extNameDetailed = "Beast Mastery"
local extSpecLimit = 253 -- Beast Mastery specId

-- Functions predeclared
local GetName
local GetNameDetailed
local GetSpells
local GetEvents
local GetSegments
local GetExtension
local SpecAllowed

local extFuncs = {
		["GetName"] = function() return GetName() end,
		["GetNameDetailed"] = function() return GetNameDetailed() end,
		["GetSpells"] = function() return GetSpells() end,
		["GetEvents"] = function() return GetEvents() end,
		["GetSegments"] = function() return GetSegments() end,
		["GetExtension"] = function() return GetExtension() end,
		["SpecAllowed"] = function(specId) return SpecAllowed(specId) end
}

-- Below lists the spells, events and segments tables.
-- The spells table lists every spell which will cause SPELL_UPDATE_COOLDOWN and SPELL_UPDATE_CHARGES events to begin a check to see if the spell's state has changed, further to a AttemptStartOnPrompt() call on success.
-- The events table lists each event which will be registered as custom to perform their segments checks. Stores the most recent passed args, and as such is initialized empty.
-- The segments table lists each of the spells and events above, for 

--- Spell Tables and Prompts --
--
-- spells[spellId] = { "Spell Name", charges, cdDur, cdExpiration, unitId, spellType}
local extSpells = { 
		[217200] = { "Barbed Shot", 	2, 	0, 	0, 	"player", 	SPELL_TYPE_ABILITY},
		[272790] = { "Frenzy", 			0, 	0, 	0, 	"pet", 		SPELL_TYPE_AURA}
}

-- events["EVENT_NAME"] = eventArgsArray (automatically generated)
local extEvents = {
	["ZONE_CHANGED_NEW_AREA"] = {},
}


local checkLen = "return select("..UNIT_AURA_DURATION..", GetAuraInfo(\"pet\", 272790)) - 5.0" -- Only declared to maintain table width -- These evaluations are to be simplified into an evaluation language, in a later version.
local extSegments = {
--[[ [spellId][i] = { {startConditionals, stopConditionals}, {seg1}, {seg2}, ..., {segN} } ]]--
		[272790] = { -- Frenzy
			{
				{
					"return spells[272790][SPELL_EXPIRATION] > 0",	-- Start conditional
					"return UnitIsDeadOrGhost('pet')"				-- Stop conditional
				},
			--- {LNGTH,		SOUND_FILE,														SndHndl,	CONDITIONAL }
				{checkLen, 	nil,															nil,		true}, -- pause until 5 seconds left on Frenzy buff.
				{5.0, 		SOUND_PATH_PREFIX..SOUNDS_ROOT.."Timers/pulse_1.ogg", 			nil,		"return select(UNIT_AURA_COUNT, GetAuraInfo(\"pet\", 272790)) == 1"},
				{nil, 		SOUND_PATH_PREFIX..SOUNDS_ROOT.."Timers/pulse_1_dropped.ogg", 	nil,		PROMPTSEG_CONDITIONAL_CONTINUATION}, -- If not refreshed, play on completion of previous segment. (Buff has dropped)
				{5.0, 		SOUND_PATH_PREFIX..SOUNDS_ROOT.."Timers/pulse_2.ogg", 			nil,		"return select(UNIT_AURA_COUNT, GetAuraInfo(\"pet\", 272790)) == 2"},
				{nil, 		SOUND_PATH_PREFIX..SOUNDS_ROOT.."Timers/pulse_2_dropped.ogg",	nil,		PROMPTSEG_CONDITIONAL_CONTINUATION},
				{5.0, 		SOUND_PATH_PREFIX..SOUNDS_ROOT.."Timers/pulse_3.ogg", 			nil,		"return select(UNIT_AURA_COUNT, GetAuraInfo(\"pet\", 272790)) == 3"},
				{nil, 		SOUND_PATH_PREFIX..SOUNDS_ROOT.."Timers/pulse_3_dropped.ogg",	nil,		PROMPTSEG_CONDITIONAL_CONTINUATION} 
			}
		},
		[217200] = { -- Barbed Shot
			{
				{
					"return spells[217200][SPELL_CHARGES] > spellsSnapshot[217200][SPELL_CHARGES]",
					false
				},
				{0.25, 		SOUND_PATH_PREFIX..SOUNDS_ROOT.."barbed_primer.ogg",			nil,		true },
				{nil, 		SOUND_PATH_PREFIX..SOUNDS_ROOT.."barbed_charge_1.ogg", 			nil,		"return GetSpellCharges(217200) == 1"},
				{nil, 		SOUND_PATH_PREFIX..SOUNDS_ROOT.."barbed_charge_2.ogg",			nil,		"return GetSpellCharges(217200) == 2"}
			}
		},
		["ZONE_CHANGED_NEW_AREA"] = {
			{
				{
					"local hasShortage = select(4, GetLFGRoleShortageRewards(1671, LFG_ROLE_SHORTAGE_RARE )) if GS.shortageResult ~= hasShortage then GS.shortageResult = hasShortage if hasShortage then return true end end return false",
					false
				},
				{1.15,		24160,										nil,		true },
				{3.5,		46849,										nil,		PROMPTSEG_CONDITIONAL_CONTINUATION },
				{nil,		PROMPTSEG_SOUND_STOP, 						nil, 		PROMPTSEG_CONDITIONAL_CONTINUATION }
			}
		},
}
--
-- /Spell Tables and Rules

--- Funcs --
--
GetName = function()
	return extName
end

GetNameDetailed = function()
	return extNameDetailed
end

GetSpells = function()
	return extSpells
end

GetEvents = function()
	return extEvents
end

GetSegments = function()
	return extSegments
end

GetExtension = function()
	return {spells=extSpells, events=extEvents, segments=extSegments}
end

SpecAllowed = function(specId)
	if extSpecLimit == ANY_SPEC_ALLOWED or extSpecLimit == specId then
		return true
	end
	return false
end
--
-- /Funcs --

-- Register Extension:
RegisterExtension(extName, extFuncs)