-- All code written and maintained by Yewchi 
-- zyewchi@gmail.com

--~ Calls out raid number of players which are low health in World or PvE, at higher frequencies for lower health. Idea theorized and developed alongside twitch.tv/BlindlyPlayingGames
--~ '1' is player's number for party and PvP. '2' to '4' are party1 to party4.
--~ '2' is focus in PvP and '3' is target in PvP.

-- Directives and advice given below is for documentation of writing further extensions.

-- --- Set these >>
local extName = "HealthMonitor"
local extNameDetailed = "Health Monitor for Healers (Raid, Party, World and PvP)"
local extShortNames = "hm|healthtracker|healthtracking"
local extSpecLimit = ANY_SPEC_ALLOWED
-- -- << Set those

local GetName
local GetNameDetailed
local GetShortNames
local GetSpells
local GetEvents
local GetSegments
local GetExtension
local SpecAllowed

local extFuncs = {
		["GetName"] = function() return GetName() end,
		["GetNameDetailed"] = function() return GetNameDetailed() end,
		["GetShortNames"] = function() return GetShortNames() end,
		["GetSpells"] = function() return GetSpells() end,
		["GetEvents"] = function() return GetEvents() end,
		["GetSegments"] = function() return GetSegments() end,
		["GetExtension"] = function() return GetExtension() end,
		["SpecAllowed"] = function(specId) return SpecAllowed(specId) end
}

-- --- Set these >>
local extSpells = {
}
	
local extEvents = {
		["UNIT_HEALTH"] = {
		},
		["ZONE_CHANGED_NEW_AREA"] = {
		},
		["GROUP_ROSTER_UPDATE"] = {
		},
		["PLAYER_SPECIALIZATION_CHANGED"] = {
		},
		["UPDATE_BATTLEFIELD_STATUS"] = {
		},
		["PLAYER_ENTERING_WORLD"] = {
		}
}

local extSegments = {
	["UNIT_HEALTH"] = {
		{
			{
				"HealthMonitor_UpdateHealthSnapshot() return false",
				false
			},
			{nil, nil, nil, nil}
		},
		{
			{
				"if GS.HM_playersCalling['player'] ~= true and (GS.HM_mode == 'party' or GS.HM_mode == 'pvp') and GS.HM_healthSnapshot['player'] < 1 then GS.HM_playersCalling['player'] = true return true end", -- Evaluated and stored as an adjustedHp (Inside HealthMonitor_UpdateHealthSnapshot()), therefore, it is < 1.0 and > 1.0. If any changes are made to the requirements e.g. "I only want to monitor those below 80% hp", then the adjusted hp can be evaluated as ((curr))/0.8
				"if GS.HM_playersCalling['player'] ~= false and ((GS.HM_mode ~= 'party' and GS.HM_mode ~= 'pvp') or GS.HM_healthSnapshot['player'] >= 1) then GS.HM_playersCalling['player'] = false return true end"
			},
			{
				"return GS.HM_delaySnapshot['player']",
				SOUND_PATH_PREFIX.."Interface/AddOns/DBM-Core/sounds/Corsica/1.ogg",
				nil,
				PROMPTSEG_CONDITIONAL_REPEATER
			} 
		},
		{
			{
				"if GS.HM_playersCalling['party1'] ~= true and GS.HM_mode == 'party' and GS.HM_healthSnapshot['party1'] < 1 then GS.HM_playersCalling['party1'] = true return true end",
				"if GS.HM_playersCalling['party1'] ~= false and (GS.HM_mode ~= 'party' or GS.HM_healthSnapshot['party1'] >= 1) then GS.HM_playersCalling['party1'] = false return true end"
			},
			{
				"return GS.HM_delaySnapshot['party1']",
				SOUND_PATH_PREFIX.."Interface/AddOns/DBM-Core/sounds/Corsica/2.ogg",
				nil,
				PROMPTSEG_CONDITIONAL_REPEATER
			} 
		},
		{
			{
				"if GS.HM_playersCalling['party2'] ~= true and GS.HM_mode == 'party' and GS.HM_healthSnapshot['party2'] < 1 then GS.HM_playersCalling['party2'] = true return true end",
				"if GS.HM_playersCalling['party2'] ~= false and (GS.HM_mode ~= 'party' or GS.HM_healthSnapshot['party2'] >= 1) then GS.HM_playersCalling['party2'] = false return true end"
			},
			{
				"return GS.HM_delaySnapshot['party2']",
				SOUND_PATH_PREFIX.."Interface/AddOns/DBM-Core/sounds/Corsica/3.ogg",
				nil,
				PROMPTSEG_CONDITIONAL_REPEATER
			} 
		},
		{
			{
				"if GS.HM_playersCalling['party3'] ~= true and GS.HM_mode == 'party' and GS.HM_healthSnapshot['party3'] < 1 then GS.HM_playersCalling['party3'] = true return true end",
				"if GS.HM_playersCalling['party3'] ~= false and (GS.HM_mode ~= 'party' or GS.HM_healthSnapshot['party3'] >= 1) then GS.HM_playersCalling['party3'] = false return true end"
			},
			{
				"return GS.HM_delaySnapshot['party3']",
				SOUND_PATH_PREFIX.."Interface/AddOns/DBM-Core/sounds/Corsica/4.ogg",
				nil,
				PROMPTSEG_CONDITIONAL_REPEATER
			} 
		},
		{
			{
				"if GS.HM_playersCalling['party4'] ~= true and GS.HM_mode == 'party' and GS.HM_healthSnapshot['party4'] < 1 then GS.HM_playersCalling['party4'] = true return true end",
				"if GS.HM_playersCalling['party4'] ~= false and (GS.HM_mode ~= 'party' or GS.HM_healthSnapshot['party4'] >= 1) then GS.HM_playersCalling['party4'] = false return true end"
			},
			{
				"return GS.HM_delaySnapshot['party4']",
				SOUND_PATH_PREFIX.."Interface/AddOns/DBM-Core/sounds/Corsica/5.ogg",
				nil,
				PROMPTSEG_CONDITIONAL_REPEATER
			} 
		},
		{
			{
				"if GS.HM_raidSegsStarted1 ~= true and GS.HM_mode == 'raid' then GS.HM_raidSegsStarted1 = true return true end",
				"if GS.HM_mode ~= 'raid' and GS.HM_raidSegsStarted1 == true then GS.HM_raidSegsStarted1 = false return true end"
			},
			{
				"if GS.HM_alertPriority[1][1] ~= nil and GS.HM_settingAlertsPriorityFlag ~= nil then return GS.HM_alertPriority[1][3] else return 0xFFFF end", -- Return delay answer, or 18 hours.
				nil,
				nil,
				true
			},
			{
				0.0,
				SOUND_FUNC_PREFIX.."if GS.HM_informingPlayerRaidN then return nil end local currTime = GetTime() GS.HM_previousCallout[GS.HM_alertPriority[1][1]] = currTime SetPromptTimestamp(currTime) return '"..SOUNDS_ROOT.."Numerical/'..(GS.HM_alertPriority[1][1])..'.ogg'",
				nil,
				PROMPTSEG_CONDITIONAL_RESTART
			}
		},
		{
			{
				"if GS.HM_raidSegsStarted2 ~= true and GS.HM_mode == 'raid' then GS.HM_raidSegsStarted2 = true return true end",
				"if GS.HM_mode ~= 'raid' and GS.HM_raidSegsStarted2 == true then GS.HM_raidSegsStarted2 = false return true end"
			},
			{
				"if GS.HM_alertPriority[2][1] ~= nil and GS.HM_settingAlertsPriorityFlag ~= nil then return GS.HM_alertPriority[2][3] else return 0xFFFF end", -- Return delay answer, or 18 hours.
				nil,
				nil,
				true
			},
			{
				0.0,
				SOUND_FUNC_PREFIX.."if GS.HM_informingPlayerRaidN then return nil end local currTime = GetTime() GS.HM_previousCallout[GS.HM_alertPriority[2][1]] = currTime SetPromptTimestamp(currTime) return '"..SOUNDS_ROOT.."Numerical/'..(GS.HM_alertPriority[2][1])..'.ogg'",
				nil,
				PROMPTSEG_CONDITIONAL_RESTART
			}
		},
		{
			{
				"if GS.HM_raidSegsStarted3 ~= true and GS.HM_mode == 'raid' then GS.HM_raidSegsStarted3 = true return true end",
				"if GS.HM_mode ~= 'raid' and GS.HM_raidSegsStarted3 == true then GS.HM_raidSegsStarted3 = false return true end"
			},
			{
				"if GS.HM_alertPriority[3][1] ~= nil and GS.HM_settingAlertsPriorityFlag ~= nil then return GS.HM_alertPriority[3][3] else return 0xFFFF end", -- Return delay answer, or 18 hours.
				nil,
				nil,
				true
			},
			{
				0.0,
				SOUND_FUNC_PREFIX.."if GS.HM_informingPlayerRaidN then return nil end local currTime = GetTime() GS.HM_previousCallout[GS.HM_alertPriority[3][1]] = currTime SetPromptTimestamp(currTime) return '"..SOUNDS_ROOT.."Numerical/'..(GS.HM_alertPriority[3][1])..'.ogg'",
				nil,
				PROMPTSEG_CONDITIONAL_RESTART
			}
		},
		{
			{
				"if GS.HM_playersCalling['focus'] ~= true and GS.HM_mode == 'pvp' and GS.HM_healthSnapshot['focus'] < 1 then GS.HM_playersCalling['focus'] = true return true end", -- Evaluated and stored as an adjustedHp (Inside HealthMonitor_UpdateHealthSnapshot()), therefore, it is < 1.0 and > 1.0. If any changes are made to the requirements e.g. "I only want to monitor those below 80% hp", then the adjusted hp can be evaluated as ((curr))/0.8
				"if GS.HM_playersCalling['focus'] ~= false and (GS.HM_mode ~= 'pvp' or GS.HM_healthSnapshot['focus'] >= 1) then GS.HM_playersCalling['focus'] = false return true end"
			},
			{
				"return GS.HM_delaySnapshot['focus']",
				SOUND_PATH_PREFIX.."Interface/AddOns/DBM-Core/sounds/Corsica/2.ogg",
				nil,
				PROMPTSEG_CONDITIONAL_REPEATER
			} 
		},
		{
			{
				"if GS.HM_playersCalling['target'] ~= true and GS.HM_mode == 'pvp' and GS.HM_healthSnapshot['target'] < 1 then GS.HM_playersCalling['target'] = true return true end", -- Evaluated and stored as an adjustedHp (Inside HealthMonitor_UpdateHealthSnapshot()), therefore, it is < 1.0 and > 1.0. If any changes are made to the requirements e.g. "I only want to monitor those below 80% hp", then the adjusted hp can be evaluated as ((curr))/0.8
				"if GS.HM_playersCalling['target'] ~= false and (GS.HM_mode ~= 'pvp' or GS.HM_healthSnapshot['target'] >= 1) then GS.HM_playersCalling['target'] = false return true end"
			},
			{
				"return GS.HM_delaySnapshot['target']",
				SOUND_PATH_PREFIX.."Interface/AddOns/DBM-Core/sounds/Corsica/3.ogg",
				nil,
				PROMPTSEG_CONDITIONAL_REPEATER
			} 
		}
	},
	["GROUP_ROSTER_UPDATE"] = {
		{
			{
				"if GS.HM_mode == 'pvp' then HealthMonitor_UpdateHealthSnapshot() elseif GS.HM_mode ~= 'party' then HealthMonitor_CheckMode('GROUP_ROSTER_UPDATE') end return false",
				false
			},
			{nil, nil, nil, nil}
		},
		{
			{
				"return GS.HM_informingPlayerRaidN == true and (GS.HM_mode ~= 'pvp' and GS.HM_mode ~= 'arena')",
				"return (GS.HM_mode ~= 'pvp' and GS.HM_mode ~= 'arena') or UnitIsUnit('player', 'raid'..GS.HM_playerRaidN)"
			},
			{
				0.75,
				SOUND_FUNC_PREFIX.."return 'Interface/AddOns/AudioQs/Sounds/Numerical/'..GS.HM_playerRaidN..'.ogg'", 
				nil, 
				true
			},
			{
				0.85,
				SOUND_PATH_PREFIX.."Interface/AddOns/AudioQs/Sounds/Numerical/your_number_is.ogg", 
				nil, 
				PROMPTSEG_CONDITIONAL_CONTINUATION
			},
			{
				nil,
				SOUND_FUNC_PREFIX.."GS.HM_informingPlayerRaidN = false return 'Interface/AddOns/AudioQs/Sounds/Numerical/'..GS.HM_playerRaidN..'.ogg'", 
				nil, 
				PROMPTSEG_CONDITIONAL_CONTINUATION
			}
		}
	},
	["ZONE_CHANGED_NEW_AREA"] = {
		{
			{
				"HealthMonitor_CheckMode('ZONE_CHANGED_NEW_AREA') return false",
				false
			},
			{nil, nil, nil, nil}
		}
	},
	["PLAYER_SPECIALIZATION_CHANGED"] = {
		{
			{
				"HealthMonitor_CheckMode('PLAYER_SPECIALIZATION_CHANGED') return false",
				false
			},
			{nil, nil, nil, nil}
		}
	},
	["PLAYER_ENTERING_WORLD"] = {
		{
			{
				"HealthMonitor_CheckMode('PLAYER_ENTERING_WORLD') HealthMonitor_UpdateHealthSnapshot() return false",
				false
			},
			{nil, nil, nil, nil}
		},
		{
			{
				"return GS.HM_informingPlayerRaidN == true",
				"return not UnitIsUnit('player', 'raid'..GS.HM_playerRaidN)"
			},
			{
				0.75,
				SOUND_FUNC_PREFIX.."return 'Interface/AddOns/AudioQs/Sounds/Numerical/'..GS.HM_playerRaidN..'.ogg'", 
				nil, 
				true
			},
			{
				0.85,
				SOUND_PATH_PREFIX.."Interface/AddOns/AudioQs/Sounds/Numerical/your_number_is.ogg", 
				nil, 
				PROMPTSEG_CONDITIONAL_CONTINUATION
			},
			{
				nil,
				SOUND_FUNC_PREFIX.."GS.HM_informingPlayerRaidN = false return 'Interface/AddOns/AudioQs/Sounds/Numerical/'..GS.HM_playerRaidN..'.ogg'", 
				nil, 
				PROMPTSEG_CONDITIONAL_CONTINUATION
			}
		}
	},
	["UPDATE_BATTLEFIELD_STATUS"] = {
		{
			{
				"HealthMonitor_CheckModePvP() return false",
				false
			},
			{nil, nil, nil, nil}
		}
	}
}
-- -- << Set those

--- Funcs --
--
GetName = function()
	return extName..extNameDetailed
end

GetNameDetailed = function()
	return extNameDetailed
end

GetSpells = function()
	return extSpells
end

GetEvents = function()
	return extEvents
end

GetSegments = function()
	return extSegments
end

GetExtension = function()
	return {spells=extSpells, events=extEvents, segments=extSegments}
end

SpecAllowed = function(specId)
	if extSpecLimit == ANY_SPEC_ALLOWED or extSpecLimit == specId then
		return true
	end
	return false
end
--
-- /Funcs --

-- Register Extension:
RegisterExtension(extName, extFuncs)

-- --- Set these >> (At will -- For detailed custom code)
local function SetGenericHpVals(unitId, setToOff)
	if UnitExists(unitId) and UnitIsPlayer(unitId) and not UnitIsDeadOrGhost(unitId) and not setToOff then
		local adjustedHp = ((UnitHealth(unitId)/UnitHealthMax(unitId))-0.1)/0.8
		GS.HM_healthSnapshot[unitId] = adjustedHp
		GS.HM_delaySnapshot[unitId] = (0.4 + 2.0*math.max(0, adjustedHp)^1.6)
	else
		GS.HM_healthSnapshot[unitId] = 1.0
		GS.HM_delaySnapshot[unitId] = 0xFFFF -- 18 hours
	end
end

function HealthMonitor_UpdateHealthSnapshot()
	local numInGroup = GetNumGroupMembers()
	
	if GS.HM_mode == "party" then
		for n=0,4,1 do
			local thisUnit = (n == 0 and "player" or "party"..n)
			SetGenericHpVals(thisUnit)
		end
	elseif GS.HM_mode == "raid" then
		HealthMonitor_CheckLowestForRaid()
	elseif GS.HM_mode == "pvp" then
		for n=1,3,1 do
			local thisUnit = (n == 3 and "target" or (n == 2 and "focus" or (n == 1 and "player")))
			if (thisUnit == "target" and 
					(UnitIsUnit(thisUnit, "player") or UnitIsUnit(thisUnit, "focus"))
			) or (thisUnit == "focus" and UnitIsUnit(thisUnit, "player")
			) or  UnitIsEnemy("player", thisUnit) then
				SetGenericHpVals(thisUnit, true) -- Set to off. 1st rule: player is only ever '1', which overrides 2nd rule: Focus is only ever '2'
			else
				SetGenericHpVals(thisUnit)
			end
		end
	end
end

function HealthMonitor_SetPreviousCallout(unitNum, timestamp)
	GS.HM_previousCallout[unitNum] = timestamp
end

function HealthMonitor_CheckMode(event)
	local instanceName, instanceType = GetInstanceInfo()
	
	if GS.HM_initialized ~= true then
		GS.HM_ALERT_UNIT_NUM = 1
		GS.HM_ALERT_HP_PERCENTAGE = 2
		GS.HM_ALERT_MAX_ALERTS = 3
		GS.HM_ASSIGNED_NUMBER_SENTENCE_PART_1 = 1
		GS.HM_ASSIGNED_NUMBER_SENTENCE_PART_2 = 2
		GS.HM_ASSIGNED_NUMBER_SENTENCE_PART_3 = 3
		GS.HM_ASSIGNED_NUMBER_SENTENCE_LENGTH_1 = 0.75
		GS.HM_ASSIGNED_NUMBER_SENTENCE_LENGTH_2 = 0.85
			  
		GS.HM_playersCalling = {}
		GS.HM_healthSnapshot = {}
		GS.HM_delaySnapshot = {}
		
		-- Raid only (Doesn't hurt having 4 empty tables for party, vs a "raidInitialized" variable checked every group_roster_update)
		GS.HM_unitIds = {} 
		GS.HM_raidUnitsIncluded = {}
		GS.HM_previousCallout = {}
		GS.HM_alertPriority = {}
			
		for n=1,GS.HM_ALERT_MAX_ALERTS,1 do
			table.insert(GS.HM_alertPriority, {nil, 100, 0xFFFF})
		end
		
		GS.HM_initialized = true
	end
    
    if (instanceType == "pvp" or instanceType == "arena") and GS.HM_mode ~= "pvp" then 
		if GS.HM_mode ~= nil then 
			print("|cff50C0F0[AudioQs]"..extensionColour.."<HealthMonitor>".."|r: "..(GS.HM_mode == "party" and "Party" or "Raid").." health call-out stopped.")
		end
		HealthMonitor_CheckModePvP()
    elseif GS.HM_instanceType ~= instanceType or event == "GROUP_ROSTER_UPDATE" or (IsInRaid() and GS.HM_mode == "party") or (not IsInRaid() and GS.HM_mode == "raid") then
        GS.HM_instanceType = instanceType
        
        GS.HM_numInGroup = (GetNumGroupMembers() > 0 and GetNumGroupMembers() or 1)
        
        if IsInRaid() then 		-- Raid mode (PvE)
			local j = 1
            for n = 1, 40, 1 do
                local thisId = "raid"..n
				GS.HM_unitIds[n] = thisId
                if UnitExists(thisId) then 
                    GS.HM_raidUnitsIncluded[j] = thisId
					j = j + 1
                end
				GS.HM_previousCallout[n] = 0
				if UnitIsUnit("player", thisId) and GS.HM_playerRaidN ~= n then
					GS.HM_informingPlayerRaidN = true
					GS.HM_playerRaidN = n
				end
            end
			
			for n=1,GetNumGroupMembers(),1 do
				local thisId = "raid"..n
				GS.HM_healthSnapshot[thisId] = 1.0
				GS.HM_delaySnapshot[thisId] = nil
			end
            
            if GS.HM_mode ~= "raid" then   
                if GS.HM_mode ~= nil then 
                    print(audioQsSpecifier..extensionColour.."<HealthMonitor>"..
						"|r: Switched to Raid health call-out.")
                else
                    print(audioQsSpecifier..extensionColour.."<HealthMonitor>"..
						"|r: Raid health call-out started.")
                end
                
                GS.HM_mode = "raid"
            end
        elseif not IsInRaid() and GS.HM_mode ~= "party" then   -- Party and World Mode (PvE)   
            GS.HM_unitIds = {"player", "party1", "party2", "party3", "party4"}
			
			GS.HM_playersCalling["player"] = false
			GS.HM_healthSnapshot["player"] = 1.0
			GS.HM_delaySnapshot["player"] = 0xFFFF
			for n = #GS.HM_unitIds - 1, 1, -1 do
				thisId = "party"..n
				GS.HM_playersCalling[thisId] = false
				GS.HM_healthSnapshot[thisId] = 1.0
				GS.HM_delaySnapshot[thisId] = 0xFFFF
			end
			
            if GS.HM_mode ~= nil then
                print(audioQsSpecifier..extensionColour.."<HealthMonitor>".."|r: Switched to Party health call-out.")
            else
                print(audioQsSpecifier..extensionColour.."<HealthMonitor>".."|r: Party health call-out started.")
            end
            
            GS.HM_mode = "party"
        end
        
		HealthMonitor_UpdateHealthSnapshot()
		
        return
    end
end

function HealthMonitor_CheckLowestForRaid() -- Raid mode
	GS.HM_settingAlertsPriorityFlag = true
	for n=1,GS.HM_ALERT_MAX_ALERTS,1 do -- TODO Read Lua scope / Garbage collection tute. May not need this.
		local thisAlertPriority = GS.HM_alertPriority[n]
		thisAlertPriority[1] = nil
		thisAlertPriority[2] = 100
		thisAlertPriority[3] = 0xFFFF
	end
	
	for i = 1, 40, 1 do
		local unitId = GS.HM_unitIds[i]
		
		if not UnitIsDeadOrGhost(unitId) and 
				GS.HM_raidUnitsIncluded[i] and
				UnitExists(unitId) then
			local unitIncHeals = math.max(0, UnitGetIncomingHeals(unitId))
			local unitHpPercentage = 
			(UnitHealth(unitId) + unitIncHeals) / UnitHealthMax(unitId)
			
			if unitHpPercentage < 0.95 then
				local n = 1
				local inserted = false
				local prevNum = nil
				local prevHp = nil
				local prevDelay = nil
				local adjustedHp = unitHpPercentage
				
				if unitId == "player" or UnitGroupRolesAssigned(unitId) == "TANK" then
					adjustedHp = math.max(0, 0.5*math.log(0.3*adjustedHp) + 1.4)
				end
				
				while n <= GS.HM_ALERT_MAX_ALERTS do
					if inserted == true then
						local nextNum = GS.HM_alertPriority[n][1]
						local nextHp = GS.HM_alertPriority[n][2]
						local nextDelay = GS.HM_alertPriority[n][3]
						
						GS.HM_alertPriority[n][1] = prevNum
						GS.HM_alertPriority[n][2] = prevHp
						GS.HM_alertPriority[n][3] = prevDelay
						
						prevNum = nextNum
						prevHp = nextHp
						prevDelay = nextDelay
					elseif adjustedHp < 
					GS.HM_alertPriority[n][2] then
						prevNum = GS.HM_alertPriority[n][1]
						prevHp = GS.HM_alertPriority[n][2]
						prevDelay = GS.HM_alertPriority[n][3]

						GS.HM_alertPriority[n][1] = i
						GS.HM_alertPriority[n][2] = adjustedHp
						GS.HM_alertPriority[n][3] = 3.0 / (1 + math.exp(-4*(math.pow(adjustedHp, 2)-0.4)))

						--print("callout for ", GS.HM_alertPriority[n][1], ": ", GS.HM_previousCallout[i])
						
						inserted = true
					end
					n = n + 1
				end
			end
		end
	end
	GS.HM_settingAlertsPriorityFlag = false
end

function HealthMonitor_CheckModePvP()
	local instanceName, instanceType, _, _, _, _, _, instanceId = GetInstanceInfo()
    
    if GS.HM_instanceCompletedOrNotBg == true then
        if GS.HM_instanceId == instanceId then
            return
        else
            GS.HM_instanceId = instanceId
            GS.HM_instanceCompletedOrNotBg = false
        end
    end
    
    if GS.HM_mode == "pvp" and 
    (GetBattlefieldWinner() ~= nil or instanceType ~= "pvp") then
        print(audioQsSpecifier..
			extensionColour..extensionColour.."<HealthMonitor>".."|r: Battleground health call-out stopped.")
        GS.HM_instanceCompletedOrNotBg = true
        GS.HM_instanceId = instanceId
        GS.HM_mode = nil
        return
    elseif (instanceType == "pvp" or instanceType == "arena") and GS.HM_mode ~= "pvp" then
        GS.HM_instanceType = instanceType
		GS.HM_instanceId = instanceId
        
        GS.HM_mode = "pvp"
		        
		HealthMonitor_UpdateHealthSnapshot()
		
        print(audioQsSpecifier..extensionColour.."<HealthMonitor>".."|r: Battleground health call-out started.")
    end
end
-- -- << Set those 