-- All code written and maintained by Yewchi 
-- zyewchi@gmail.com

--~ Table generation; PromptTicker and segment interpretation

--- Initialization --
--
------- Flags --
--
PROMPT_STAGE_OFF = 									0x0

PROMPTSEG_CONDITIONAL_USE_PREVIOUS =				0xFFF0
PROMPTSEG_CONDITIONAL_USE_STOP =					0xFFF1
PROMPTSEG_CONDITIONAL_CONTINUATION =				0xFFF2
PROMPTSEG_CONDITIONAL_REPEATER =					0xFFF3
PROMPTSEG_CONDITIONAL_RESTART =						0xFFF4

PROMPTSEG_SOUND_STOP =								0xEFF0
--
------ /Flags --


------- Table key Reference --
--
SEGT_CONDITIONALS =	1
SEGT_FIRST = 2
SEGT_CONDITIONALS_START = 1
SEGT_CONDITIONALS_STOP = 2
SEGT_INIT_FUNC = 1
SEGT_SEGMENT_KEY =										0xFFFF -- Trusting lua's " #tbl "
SEGT_SEGMENT_INDEX = 									0xFFFE

PROMPT_TIMESTAMP = 1
PROMPT_CURR_STAGE = 2
PROMPT_SEGMENT_KEY = 3
PROMPT_SEGMENT_TABLE_INDEX = 4

PROMPTSEG_LENGTH = 1
PROMPTSEG_SOUND = 2
PROMPTSEG_HANDLE = 3
PROMPTSEG_CONDITIONAL = 4

REG_PROMPT_INDEX = 1
REG_SEGMENT_KEY = 2
REG_SEGMENT_INDEX = 3

SPELL_INFO_NAME = 1
SPELL_INFO_ID = 7
--
------ /Table key Reference --

------- Static vals --
--
AUDIO_CHANNEL = "DIALOG"

TICKER_SKIP_FRAMES = 3 -- TODO Cheap optimisation, should be evaluates n = [1,2,3]; n = n+3 of prompts per frame.
--
------ /Static vals --

------- AddOn variables --
--
local spellsLookUp = {}

local unitsIncluded = {}

local promptsTable = {}
local segmentPromptIndex = {}

local segments = {}

local tickerStep = 0

local registered = {} -- Prompts are registered when they're being checked in the prompt ticker, allows for extensions to edit their own prompt data.

Frame_PromptTicker = CreateFrame("Frame", "Prompt Ticker")
--
------ /AddOn variables --
--
-- /Initialization --

--- Funcs --
--
local AttemptStartPrompts
local PromptTicker

-------------- GenerateTablesAndIndices()
local function GenerateTablesAndIndices()
	local spells = GSI_GetSpellsTable()
	local abilityTable = {}
	local auraTable = {}
	do
		local i = 1
		local j = 1
		for spellId,spell in pairs(spells) do
			-- spellsLookUp
			spellsLookUp[spell[SPELL_SPELL_NAME]] = spellId
			-- abilityTable
			local previousAbilitySpellId = (#abilityTable > 0 and abilityTable[i-1] or nil)
			if spell[SPELL_SPELL_TYPE] == SPELL_TYPE_ABILITY and previousAbilitySpellId ~= spellId then
				abilityTable[i] = spellId
				i = i + 1
			end
			-- auraTable
			local previousAuraSpellId = (#auraTable > 0 and auraTable[j-1] or nil)
			if spell[SPELL_SPELL_TYPE] == SPELL_TYPE_AURA and previousAuraSpellId ~= spellId then
				auraTable[j] = spellId
				j = j + 1
			end
		end
	end
	do
		local i = 1
		for k,array in pairs(segments) do
			segmentPromptIndex[k] = {}
			for n=1,#array,1 do
				if #array[n] >= SEGT_FIRST then -- Is this a true set of segments for a prompt? Not an initialization function?
					-- promptsTable
					promptsTable[i] = {	0, PROMPT_STAGE_OFF, k, n } -- PROMPT_TIMESTAMP, PROMPT_CURR_STAGE, PROMPT_SEGMENT_KEY, PROMPT_SEGMENT_TABLE_INDEX
					-- segmentPromptIndex
					segmentPromptIndex[k][n] = i
					--segments[k][SEGT_SEGMENT_KEY] = k -- Key and Index required for editing prompt data from inside segment code, can be useful.
					--segments[k][SEGT_SEGMENT_INDEX] = i 
					i = i + 1
				end
			end
		end
	end
	-- unitsIncluded
	for n=1,#promptsTable,1 do
		local thisSegmentKey = promptsTable[n][PROMPT_SEGMENT_KEY]
		local thisSegmentArrayIndex = promptsTable[n][PROMPT_SEGMENT_TABLE_INDEX]
		if thisSegmentArrayIndex == 1 then
			if type(thisSegmentKey) == "number" then 
				local thisUnit = spells[thisSegmentKey][SPELL_UNIT_ID]
				if thisUnit ~= nil then
					if #unitsIncluded == 0 then
						unitsIncluded[#unitsIncluded+1] = thisUnit
					else
						local insert = true
						for n=1,#unitsIncluded,1 do
							if unitsIncluded[n] == thisUnit then
								insert = false
								break
							end
						end
						if insert then
							unitsIncluded[#unitsIncluded+1] = thisUnit
						end
					end
				end
			end
		end
	end
	
	return abilityTable, auraTable
end

-- Needs confirmation, error thrown on missing file
-------------- PlaySoundGetHandle()
local function PlaySoundGetHandle(sound)
	if hushMode == true then
		return nil
	end

	eval = GSI_EvaluateSound(sound)
	
	if eval == nil then
		return nil
	end
	
	if type(eval) == "number" then
if VERBOSE then print(audioQsSpecifier..debugSpecifier.."Playing \""..eval.."\"") end
		return select(2, PlaySound(eval, AUDIO_CHANNEL))
	elseif type(eval) == "string" then
		return select(2, PlaySoundFile(eval, AUDIO_CHANNEL))
	end
	error({code=ERR_INVALID_SOUND_DATA, func="PlaySoundGetHandle(sound="..(sound~=nil and sound or "nil")..")"})
end

-------------- StopAllSegmentsSounds()
local function StopAllSegmentsSounds(segmentsArray)
	for n=SEGT_FIRST,#segmentsArray,1 do
		local soundHandle = segmentsArray[n][PROMPTSEG_HANDLE]
		if soundHandle ~= nil then
			StopSound(soundHandle)
			segmentsArray[n][PROMPTSEG_HANDLE] = nil
		end
	end
end

-------------- CheckStopSegments()
local function CheckStopSegments(segmentsArray, prompt, allowContinuationOfPlay, forceStop)
	if prompt[PROMPT_CURR_STAGE] < SEGT_FIRST then
		if prompt[PROMPT_SEGMENT_INDEX] == 2 then print(PrintableTable(prompt)) end
		return false
	end
	if GSI_EvaluateConditional(segmentsArray[SEGT_CONDITIONALS][SEGT_CONDITIONALS_STOP]) or forceStop == true then
		if allowContinuationOfPlay == false then
			StopAllSegmentsSounds(segmentsArray)
		end
		prompt[PROMPT_TIMESTAMP] = 0
		prompt[PROMPT_CURR_STAGE] = PROMPT_STAGE_OFF
		--print("CheckStopSegments():", prompt[PROMPT_SEGMENT_KEY], prompt[PROMPT_SEGMENT_TABLE_INDEX], "T,", GS.HM_mode)
		return true
	end
		--print("CheckStopSegments():", prompt[PROMPT_SEGMENT_KEY], prompt[PROMPT_SEGMENT_TABLE_INDEX], "F,", GS.HM_mode)
	return false
end

-------------- GetNextTrueSegment()
local function GetNextTrueSegment(segmentsArray, nextPromptStage)
	if nextPromptStage > #segmentsArray and nextPromptStage > SEGT_FIRST then
		if segmentsArray[nextPromptStage-1][PROMPTSEG_CONDITIONAL] == PROMPTSEG_CONDITIONAL_REPEATER then
			return nextPromptStage-1
		elseif segmentsArray[nextPromptStage-1][PROMPTSEG_CONDITIONAL] == PROMPTSEG_CONDITIONAL_RESTART then
			return SEGT_FIRST
		end
		return PROMPT_STAGE_OFF
	end
	
	local skippingFalseConditionalSet = false
	for n=nextPromptStage,#segmentsArray,1 do -- Search downards for true conditionals
		local conditional = segmentsArray[n][PROMPTSEG_CONDITIONAL]
		if type(conditional) == "string" then
			if GSI_EvaluateConditional(conditional) then 
				return n
			end
			skippingFalseConditionalSet = true
		elseif conditional == PROMPTSEG_CONDITIONAL_USE_PREVIOUS then
			if skippingFalseConditionalSet == false then
				for p=n-1,SEGT_FIRST,-1 do -- search upwards for this conditional set's validity
					local prevConditional = segmentsArray[p][PROMPTSEG_CONDITIONAL]
					if prevConditional == nil then 
						return PROMPT_STAGE_OFF -- TODO if p ~= SEGT_FIRST then error(SEGT_MALFORMED_SEGMENT_TABLE) ?
					end
					
					if prevConditional ~= PROMPTSEG_CONDITIONAL_USE_PREVIOUS then
						if GSI_EvaluateConditional(prevConditional) then
							return n
						end
						skippingFalseConditionalSet = true
						break
					end
				end
			end
		elseif conditional == PROMPTSEG_CONDITIONAL_USE_STOP then 
			if GSI_EvaluateConditional(segmentsArray[SEGT_CONDITIONALS][SEGT_CONDITIONALS_STOP]) then
				return n
			end
			skippingFalseConditionalSet = true
		elseif conditional == PROMPTSEG_CONDITIONAL_CONTINUATION and skippingFalseConditionalSet == false then
			return n
		elseif conditional == PROMPTSEG_CONDITIONAL_REPEATER then
			return math.max(SEGT_FIRST, n-1)
		elseif conditional == PROMPTSEG_CONDITIONAL_RESTART then
			return n
		else
			if GSI_EvaluateConditional(conditional) then
				return n
			end
			skippingFalseConditionalSet = true
		end
		if segmentsArray[n][PROMPTSEG_LENGTH] == nil and type(segmentsArray[n+1]) ~= "table" then
			break
		end
	end
	return PROMPT_STAGE_OFF
end

-------------- RegisterPrompt()
local function RegisterPrompt(promptsTableIndex)
	if type(promptsTableIndex) ~= "number" then if DEBUG then print(audioQsSpecifier..debugSpecifier.."invalid access request to promptsTable in RegisterPrompt(promptsTableIndex = ", promptsTableIndex, ")") end return nil end
	local prompt = promptsTable[promptsTableIndex]
	if prompt == nil then if DEBUG then print(audioQsSpecifier..debugSpecifier.."invalid access request to promptsTable in RegisterPrompt(promptsTableIndex = ", promptsTableIndex, ")") end return nil end

	registered[REG_PROMPT_INDEX] = promptsTableIndex
	registered[REG_SEGMENT_KEY] = promptsTable[PROMPT_SEGMENT_KEY]
	registered[REG_SEGMENT_INDEX] = promptsTable[PROMPT_SEGMENT_INDEX]
		
	beenSet = true
		
	return unpack(promptsTable[promptsTableIndex])
end

-------- SetPromptTimestamp()
function SetPromptTimestamp(timestamp)
	promptsTable[registered[REG_PROMPT_INDEX]][PROMPT_TIMESTAMP] = timestamp
end

-------- GetPromptStage()
function GetPromptStage()
	return promptsTable[registered[REG_PROMPT_INDEX]][PROMPT_CURR_STAGE]
end

----- Continues the running of PromptTicker if any prompts are in-progress.
-------------- CheckPrompts()
function CheckPrompts()
	for n=#promptsTable,1,-1 do
		if promptsTable[n][PROMPT_CURR_STAGE] ~= PROMPT_STAGE_OFF then
			if Frame_PromptTicker:GetScript("OnUpdate") == nil then
				Frame_PromptTicker:SetScript("OnUpdate", function() tickerStep = tickerStep + 1 if tickerStep < TICKER_SKIP_FRAMES then return else tickerStep = 0 end local success, err = pcall(PromptTicker) if not success then HandleError(err, "[OnUpdate]", "PromptTicker()") end end)
			end
			return
		end
	end
	if Frame_PromptTicker:GetScript("OnUpdate") ~= nil then
		Frame_PromptTicker:SetScript("OnUpdate", nil)
	end
end

-------- AttemptStartPrompt()
function AttemptStartPrompt(spellId, runCheckPrompts)
	local segmentPromptsArray = segmentPromptIndex[spellId]
	if segmentPromptsArray == nil then
		error({code=ERR_UNKNOWN_SPELL_AS_ARGUMENT, func="AttemptStartPrompt(spellId="..(spellId~=nil and spellId or "nil").." runCheckPrompts="..(runCheckPrompts~=nil and runCheckPrompts or "nil")..")"})
	end
	
	for n=1,#segmentPromptsArray,1 do -- For each set of segments pertaining to this spellId
		local thisSegments = segments[spellId][n]
		local thisSegmentsPrompt = promptsTable[segmentPromptsArray[n]]

		if GSI_EvaluateConditional(thisSegments[SEGT_CONDITIONALS][SEGT_CONDITIONALS_START]) then
			CheckStopSegments(thisSegments, thisSegmentsPrompt, false, true) -- TODO--Design decis. Always stop if the spell has been updated?
			
			local startingSegment = GetNextTrueSegment(thisSegments, SEGT_FIRST)
			
			thisSegmentsPrompt[PROMPT_TIMESTAMP] = GetTime()
			thisSegmentsPrompt[PROMPT_CURR_STAGE] = startingSegment
			 
			thisSegments[startingSegment][PROMPTSEG_HANDLE] = PlaySoundGetHandle(thisSegments[startingSegment][PROMPTSEG_SOUND])
		end
		CheckStopSegments(thisSegments, thisSegmentsPrompt, false)
		if runCheckPrompts ~= false then 
			CheckPrompts()
		end
	end
end

-- TODO Needs efficiency if it's just an initialization prompt. 
-------- AttemptStartPromptOnEvent()
function AttemptStartPromptOnEvent(event, runCheckPrompts)
	local segmentPromptsArray = segmentPromptIndex[event]
	if segmentPromptsArray == nil then
		error({code=ERR_UNKNOWN_EVENT_AS_ARGUMENT, func="AttemptStartPromptOnEvent(event="..(event~=nil and event or "nil").." runCheckPrompts="..(runCheckPrompts~=nil and runCheckPrompts or "nil")..")"})
	end
	
	for n=1,#segmentPromptsArray,1 do -- For each set of segments pertaining to this event
		local thisSegments = segments[event][n]
		
		local thisSegmentsPrompt = promptsTable[segmentPromptsArray[n]]
		
		if GSI_EvaluateConditional(thisSegments[SEGT_CONDITIONALS][SEGT_CONDITIONALS_START]) then
			CheckStopSegments(thisSegments, thisSegmentsPrompt, false, true) -- TODO--Design decis. Always stop if the spell has been updated?
		
			local startingSegment = GetNextTrueSegment(thisSegments, SEGT_FIRST)
		
			thisSegmentsPrompt[PROMPT_TIMESTAMP] = GetTime()
			thisSegmentsPrompt[PROMPT_CURR_STAGE] = startingSegment
			
			thisSegments[startingSegment][PROMPTSEG_HANDLE] = PlaySoundGetHandle(thisSegments[startingSegment][PROMPTSEG_SOUND])
		end
		CheckStopSegments(thisSegments, thisSegmentsPrompt, false)
		if runCheckPrompts ~= false then
			CheckPrompts()
		end
	end
end

-------- UnitIsIncluded()
function UnitIsIncluded(unitId)
	for n=1,#unitsIncluded,1 do
		if unitsIncluded[n] == unitId then
			return true
		end
	end
	return false
end

-------- InitializePrompts()
function InitializePrompts(specIdToLoad)
	segments = GSI_GetSegmentsTable(specIdToLoad)
	if segments == nil then
		return nil
	end

	return GenerateTablesAndIndices()
end

--
-- /Funcs --

--- Event Funcs --
--
-- Should register prompts instead of iteration over promptsTable. Regardless, CPU utilization still low.
-------------- PromptTicker()
PromptTicker = function()
	local currTime = GetTime()
	for promptsTableIndex=1,#promptsTable,1 do
		if promptsTable[promptsTableIndex][PROMPT_CURR_STAGE] > PROMPT_STAGE_OFF then
			local promptTimestamp, promptStage, segmentKey, segmentsArrayIndex = RegisterPrompt(promptsTableIndex)
			local thisSegments = segments[segmentKey][segmentsArrayIndex]
			if not CheckStopSegments(thisSegments, promptsTable[promptsTableIndex], false) then
				local promptLength = GSI_EvaluateLength(thisSegments[promptStage][PROMPTSEG_LENGTH])
				if currTime > promptLength + promptTimestamp then
					local nextPromptStage = GetNextTrueSegment(thisSegments, promptStage+1)
					local stopSegments = nextPromptStage == PROMPT_STAGE_OFF
					local nextSegment = thisSegments[nextPromptStage]
				
					if not stopSegments then
						local segmentSound = nextSegment[PROMPTSEG_SOUND]
						if segmentSound == PROMPTSEG_SOUND_STOP then
							StopAllSegmentsSounds(thisSegments)
						else
							nextSegment[PROMPTSEG_HANDLE] = PlaySoundGetHandle(segmentSound)
						end
						
						promptsTable[promptsTableIndex][PROMPT_TIMESTAMP] = currTime
						
						if nextSegment[PROMPTSEG_LENGTH] ~= nil then
							promptsTable[promptsTableIndex][PROMPT_CURR_STAGE] = nextPromptStage
						else
							stopSegments = true
						end
					end
					
					if stopSegments then
						CheckStopSegments(thisSegments, promptsTable[promptsTableIndex], true, true)
						CheckPrompts()
					end
				end
			end
		end
	end
end
--
-- /Event Funcs