-- All code written and maintained by Yewchi
-- zyewchi@gmail.com

--~ Top-level processing of events in relation to prompts and the GameStateInterface; Interfaces the loading of specialization data.

-- Needs further abstraction.

--- Initialization --
--
------- Table key reference --
--
COMBAT_LOG_SOURCE_GUID = 4
COMBAT_LOG_SPELL_ID = 12
--
------ /Table key reference --

------- AddOn variables --
--
local abilityTable = {}
local auraTable = {}
--
------ /AddOn variables --
--
-- /Initialization --

--- Funcs --
--
function SpecHasPrompts(specIdToCheck)
	return GSI_SpecHasPrompts(specIdToCheck)
end

function SetAbilityAndAuraTables(newAbilities, newAuras)
	abilityTable = newAbilities
	auraTable = newAuras
end

-------- FindPromptsFromUnitAura()
function FindPromptsFromUnitAura(aura)
	if aura[UNIT_AURA_TIME_MOD] ~= AURA_ALWAYS_IS_VALUE then
		error({code=ERR_INVALID_AURA_DATA, func="FindPromptsFromUnitAura(aura=", unpack(aura), ")"})
	end
	
	if GSI_AuraIsIncluded(aura[UNIT_AURA_SPELL_ID]) then 
		return ProcessAuraForPrompts(unpack(aura))
	end
end

-------- ProcessCustomEventForPrompts()
function ProcessCustomEventForPrompts(event, ...)
	if GSI_EventIsIncluded(event) then
		GSI_UpdateEventTable(event, ...)
		AttemptStartPromptOnEvent(event)
	end
end

-------- ProcessCombatLogForPrompts()
function ProcessCombatLogForPrompts()
	local combatLogEventInfo = {CombatLogGetCurrentEventInfo()}
	local spellId = combatLogEventInfo[COMBAT_LOG_SPELL_ID]
	local spell = GSI_GetSpell(spellId)
	
	if spell == nil or combatLogEventInfo[COMBAT_LOG_SOURCE_GUID] ~= PLAYER_GUID then
		return
	end
	
	if spell[SPELL_SPELL_TYPE] == SPELL_TYPE_ABILITY then
		local cdStart, cdDur = GetSpellCooldown(spellId)
		local cdExpiration = cdStart + cdDur
		
		if GSI_UpdateSpellTable(spellId, cdDur, cdExpiration) then
			AttemptStartPrompt(spellId)
		else
			CheckPrompts()
		end
	end
end

-------- ProcessSpellCooldownForPrompts()
function ProcessSpellCooldownsForPrompts()
	for n=1,#abilityTable,1 do
		local spellId = abilityTable[n]
		local cdStart, cdDur = GetSpellCooldown(spellId)
		local cdExpiration = cdStart + cdDur
		
		if GSI_UpdateSpellTable(spellId, cdDur, cdExpiration) then
			AttemptStartPrompt(spellId)
		else
			CheckPrompts()
		end
	end
end

-- Puting full aura table processing in here, then call this func, such that
--   we don't require passing table spell refs to core.
-------- ProccessAuraForPrompts()
function ProcessAuraForPrompts(...)
	if ... == nil then
		error({code=ERR_INVALID_ARGS, func="ProcessAuraForPrompts(...="..(...~=nil and ... or "nil")..")"})
	end

	local name, _, count, _, cdDur, cdExpiration, _, _, _, spellId = ...
	local spellData = GSI_GetSpell(spellId)
	
	if spellData == nil then 
		error({code=ERR_UNKNOWN_SPELL_AS_ARGUMENT, func="ProcessAuraForPrompts(...="..(...~=nil and ... or "nil")..")"})
	elseif GSI_UpdateSpellTable(spellId, cdDur, cdExpiration) then
		AttemptStartPrompt(spellId)
	end
end

-------- LoadSpecialization()
function LoadSpecialization()
	specId = GetSpec()
	local funcsForLoading = nil -- Needs evaluation of some sort.
	
	if GSI_LoadSpecTables(specId, funcsForLoading) then
if DEBUG then print(audioQsSpecifier..debugSpecifier.."Loaded specialization tables.") end
		abilityTable, auraTable = InitializePrompts(specId)
		GSI_UpdateAllSpellTables()
if DEBUG then print(audioQsSpecifier..debugSpecifier.."Prompt tables generated. Spell info updated.") end
	end
end
--
-- /Funcs --